@@ -28,11 +28,15 @@
 #include "device-private.h"
 #include "array.h"
 #include "rbstream.h"
+#include "platform.h"
 
 #define C_ARRAY_SIZE(array) (sizeof (array) / sizeof *(array))
 
 #define ISINSTANCE(device) dc_device_isinstance((device), &mares_iconhd_device_vtable)
 
+#define NSTEPS    1000
+#define STEP(i,n) (NSTEPS * (i) / (n))
+
 #define MATRIX    0x0F
 #define SMART      0x000010
 #define SMARTAPNEA 0x010010
@@ -40,13 +44,30 @@
 #define ICONHDNET 0x15
 #define PUCKPRO   0x18
 #define NEMOWIDE2 0x19
+#define GENIUS    0x1C
 #define PUCK2     0x1F
 #define QUADAIR   0x23
 #define SMARTAIR  0x24
 #define QUAD      0x29
 
+#define MAXRETRIES 4
+
 #define ACK 0xAA
 #define EOF 0xEA
+#define XOR 0xA5
+
+#define CMD_VERSION   0xC2
+#define CMD_FLASHSIZE 0xB3
+#define CMD_READ      0xE7
+#define CMD_OBJ_INIT  0xBF
+#define CMD_OBJ_EVEN  0xAC
+#define CMD_OBJ_ODD   0xFE
+
+#define OBJ_LOGBOOK       0x2008
+#define OBJ_LOGBOOK_COUNT 0x01
+#define OBJ_DIVE          0x3000
+#define OBJ_DIVE_HEADER   0x02
+#define OBJ_DIVE_DATA     0x03
 
 #define AIR       0
 #define GAUGE     1
@@ -69,10 +90,13 @@ typedef struct mares_iconhd_device_t {
 	dc_iostream_t *iostream;
 	const mares_iconhd_layout_t *layout;
 	unsigned char fingerprint[10];
+	unsigned int fingerprint_size;
 	unsigned char version[140];
 	unsigned int model;
 	unsigned int packetsize;
-	unsigned int splitcommand;
+	unsigned char cache[20];
+	unsigned int available;
+	unsigned int offset;
 } mares_iconhd_device_t;
 
 static dc_status_t mares_iconhd_device_set_fingerprint (dc_device_t *abstract, const unsigned char data[], unsigned int size);
@@ -127,6 +151,7 @@ mares_iconhd_get_model (mares_iconhd_device_t *device)
 		{"Icon AIR",    ICONHDNET},
 		{"Puck Pro",    PUCKPRO},
 		{"Nemo Wide 2", NEMOWIDE2},
+		{"Genius",      GENIUS},
 		{"Puck 2",      PUCK2},
 		{"Quad Air",    QUADAIR},
 		{"Smart Air",   SMARTAIR},
@@ -147,23 +172,92 @@ mares_iconhd_get_model (mares_iconhd_device_t *device)
 }
 
 static dc_status_t
-mares_iconhd_transfer (mares_iconhd_device_t *device,
+mares_iconhd_read (mares_iconhd_device_t *device, unsigned char data[], size_t size)
+{
+	dc_status_t rc = DC_STATUS_SUCCESS;
+	dc_transport_t transport = dc_iostream_get_transport(device->iostream);
+
+	size_t nbytes = 0;
+	while (nbytes < size) {
+		if (transport == DC_TRANSPORT_BLE) {
+			if (device->available == 0) {
+				// Read a packet into the cache.
+				size_t len = 0;
+				rc = dc_iostream_read (device->iostream, device->cache, sizeof(device->cache), &len);
+				if (rc != DC_STATUS_SUCCESS)
+					return rc;
+
+				device->available = len;
+				device->offset = 0;
+			}
+		}
+
+		// Set the minimum packet size.
+		size_t length = (transport == DC_TRANSPORT_BLE) ? device->available : size - nbytes;
+
+		// Limit the packet size to the total size.
+		if (nbytes + length > size)
+			length = size - nbytes;
+
+		if (transport == DC_TRANSPORT_BLE) {
+			// Copy the data from the cached packet.
+			memcpy (data + nbytes, device->cache + device->offset, length);
+			device->available -= length;
+			device->offset += length;
+		} else {
+			// Read the packet.
+			rc = dc_iostream_read (device->iostream, data + nbytes, length, &length);
+			if (rc != DC_STATUS_SUCCESS)
+				return rc;
+		}
+
+		nbytes += length;
+	}
+
+	return rc;
+}
+
+static dc_status_t
+mares_iconhd_write (mares_iconhd_device_t *device, const unsigned char data[], size_t size)
+{
+	dc_status_t rc = DC_STATUS_SUCCESS;
+	dc_transport_t transport = dc_iostream_get_transport(device->iostream);
+
+	size_t nbytes = 0;
+	while (nbytes < size) {
+		// Set the maximum packet size.
+		size_t length = (transport == DC_TRANSPORT_BLE) ? sizeof(device->cache) : size - nbytes;
+
+		// Limit the packet size to the total size.
+		if (nbytes + length > size)
+			length = size - nbytes;
+
+		// Write the packet.
+		rc = dc_iostream_write (device->iostream, data + nbytes, length, &length);
+		if (rc != DC_STATUS_SUCCESS)
+			return rc;
+
+		nbytes += length;
+	}
+
+	return rc;
+}
+
+static dc_status_t
+mares_iconhd_packet (mares_iconhd_device_t *device,
 	const unsigned char command[], unsigned int csize,
 	unsigned char answer[], unsigned int asize)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
 	dc_device_t *abstract = (dc_device_t *) device;
-	unsigned int split_csize;
 
 	assert (csize >= 2);
 
 	if (device_is_cancelled (abstract))
 		return DC_STATUS_CANCELLED;
 
-	split_csize = device->splitcommand ? 2 : csize;
-
 	// Send the command header to the dive computer.
-	status = dc_iostream_write (device->iostream, command, split_csize, NULL);
+	status = mares_iconhd_write (device, command, 2);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to send the command.");
 		return status;
@@ -171,7 +265,7 @@ mares_iconhd_transfer (mares_iconhd_device_t *device,
 
 	// Receive the header byte.
 	unsigned char header[1] = {0};
-	status = dc_iostream_read (device->iostream, header, sizeof (header), NULL);
+	status = mares_iconhd_read (device, header, sizeof (header));
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the answer.");
 		return status;
@@ -183,9 +277,9 @@ mares_iconhd_transfer (mares_iconhd_device_t *device,
 		return DC_STATUS_PROTOCOL;
 	}
 
-	// Send any remaining command payload to the dive computer.
-	if (csize > split_csize) {
-		status = dc_iostream_write (device->iostream, command + split_csize, csize - split_csize, NULL);
+	// Send the command payload to the dive computer.
+	if (csize > 2) {
+		status = mares_iconhd_write (device, command + 2, csize - 2);
 		if (status != DC_STATUS_SUCCESS) {
 			ERROR (abstract->context, "Failed to send the command.");
 			return status;
@@ -193,7 +287,7 @@ mares_iconhd_transfer (mares_iconhd_device_t *device,
 	}
 
 	// Read the packet.
-	status = dc_iostream_read (device->iostream, answer, asize, NULL);
+	status = mares_iconhd_read (device, answer, asize);
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the answer.");
 		return status;
@@ -201,7 +295,7 @@ mares_iconhd_transfer (mares_iconhd_device_t *device,
 
 	// Receive the trailer byte.
 	unsigned char trailer[1] = {0};
-	status = dc_iostream_read (device->iostream, trailer, sizeof (trailer), NULL);
+	status = mares_iconhd_read (device, trailer, sizeof (trailer));
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to receive the answer.");
 		return status;
@@ -216,6 +310,140 @@ mares_iconhd_transfer (mares_iconhd_device_t *device,
 	return DC_STATUS_SUCCESS;
 }
 
+static dc_status_t
+mares_iconhd_transfer (mares_iconhd_device_t *device, const unsigned char command[], unsigned int csize, unsigned char answer[], unsigned int asize)
+{
+	unsigned int nretries = 0;
+	dc_status_t rc = DC_STATUS_SUCCESS;
+	while ((rc = mares_iconhd_packet (device, command, csize, answer, asize)) != DC_STATUS_SUCCESS) {
+		// Automatically discard a corrupted packet,
+		// and request a new one.
+		if (rc != DC_STATUS_PROTOCOL && rc != DC_STATUS_TIMEOUT)
+			return rc;
+
+		// Abort if the maximum number of retries is reached.
+		if (nretries++ >= MAXRETRIES)
+			return rc;
+
+		// Discard any garbage bytes.
+		dc_iostream_purge (device->iostream, DC_DIRECTION_INPUT);
+	}
+
+	return DC_STATUS_SUCCESS;
+}
+
+static dc_status_t
+mares_iconhd_read_object(mares_iconhd_device_t *device, dc_event_progress_t *progress, dc_buffer_t *buffer, unsigned int index, unsigned int subindex)
+{
+	dc_status_t status = DC_STATUS_SUCCESS;
+	dc_device_t *abstract = (dc_device_t *) device;
+	dc_transport_t transport = dc_iostream_get_transport (device->iostream);
+	const unsigned int maxpacket = (transport == DC_TRANSPORT_BLE) ? 124 : 504;
+
+	// Update and emit a progress event.
+	unsigned int initial = 0;
+	if (progress) {
+		initial = progress->current;
+		device_event_emit (abstract, DC_EVENT_PROGRESS, progress);
+	}
+
+	// Transfer the init packet.
+	unsigned char rsp_init[16];
+	unsigned char cmd_init[18] = {
+		CMD_OBJ_INIT,
+		CMD_OBJ_INIT ^ XOR,
+		0x40,
+		(index >> 0) & 0xFF,
+		(index >> 8) & 0xFF,
+		subindex & 0xFF
+	};
+	memset (cmd_init + 6, 0x00, sizeof(cmd_init) - 6);
+	status = mares_iconhd_transfer (device, cmd_init, sizeof (cmd_init), rsp_init, sizeof (rsp_init));
+	if (status != DC_STATUS_SUCCESS) {
+		ERROR (abstract->context, "Failed to transfer the init packet.");
+		return status;
+	}
+
+	// Verify the packet header.
+	if (memcmp (cmd_init + 3, rsp_init + 1, 3) != 0) {
+		ERROR (abstract->context, "Unexpected packet header.");
+		return DC_STATUS_PROTOCOL;
+	}
+
+	unsigned int nbytes = 0, size = 0;
+	if (rsp_init[0] == 0x41) {
+		// A large (and variable size) payload is split into multiple
+		// data packets. The first packet contains only the total size
+		// of the payload.
+		size = array_uint32_le (rsp_init + 4);
+	} else if (rsp_init[0] == 0x42) {
+		// A short (and fixed size) payload is embedded into the first
+		// data packet.
+		size = sizeof(rsp_init) - 4;
+
+		// Append the payload to the output buffer.
+		if (!dc_buffer_append (buffer, rsp_init + 4, sizeof(rsp_init) - 4)) {
+			ERROR (abstract->context, "Insufficient buffer space available.");
+			return DC_STATUS_NOMEMORY;
+		}
+
+		nbytes += sizeof(rsp_init) - 4;
+	} else {
+		ERROR (abstract->context, "Unexpected packet type (%02x).", rsp_init[0]);
+		return DC_STATUS_PROTOCOL;
+	}
+
+	// Update and emit a progress event.
+	if (progress) {
+		progress->current = initial + STEP (nbytes, size);
+		device_event_emit (abstract, DC_EVENT_PROGRESS, progress);
+	}
+
+	unsigned int npackets = 0;
+	while (nbytes < size) {
+		// Get the command byte.
+		unsigned char toggle = npackets % 2;
+		unsigned char cmd = toggle == 0 ? CMD_OBJ_EVEN : CMD_OBJ_ODD;
+
+		// Get the packet size.
+		unsigned int len = size - nbytes;
+		if (len > maxpacket) {
+			len = maxpacket;
+		}
+
+		// Transfer the segment packet.
+		unsigned char rsp_segment[1 + 504];
+		unsigned char cmd_segment[] = {cmd, cmd ^ XOR};
+		status = mares_iconhd_transfer (device, cmd_segment, sizeof (cmd_segment), rsp_segment, len + 1);
+		if (status != DC_STATUS_SUCCESS) {
+			ERROR (abstract->context, "Failed to transfer the segment packet.");
+			return status;
+		}
+
+		// Verify the packet header.
+		if ((rsp_segment[0] & 0xF0) >> 4 != toggle) {
+			ERROR (abstract->context, "Unexpected packet header (%02x).", rsp_segment[0]);
+			return DC_STATUS_PROTOCOL;
+		}
+
+		// Append the payload to the output buffer.
+		if (!dc_buffer_append (buffer, rsp_segment + 1, len)) {
+			ERROR (abstract->context, "Insufficient buffer space available.");
+			return DC_STATUS_NOMEMORY;
+		}
+
+		nbytes += len;
+		npackets++;
+
+		// Update and emit the progress events.
+		if (progress) {
+			progress->current = initial + STEP (nbytes, size);
+			device_event_emit (abstract, DC_EVENT_PROGRESS, progress);
+		}
+	}
+
+	return status;
+}
 
 dc_status_t
 mares_iconhd_device_open (dc_device_t **out, dc_context_t *context, dc_iostream_t *iostream)
@@ -237,18 +465,13 @@ mares_iconhd_device_open (dc_device_t **out, dc_context_t *context, dc_iostream_
 	device->iostream = iostream;
 	device->layout = NULL;
 	memset (device->fingerprint, 0, sizeof (device->fingerprint));
+	device->fingerprint_size = sizeof (device->fingerprint);
 	memset (device->version, 0, sizeof (device->version));
 	device->model = 0;
 	device->packetsize = 0;
-
-	/*
-	 * At least the Mares Matrix needs the command to be split into
-	 * base and argument, with a wait for the ACK byte in between.
-	 *
-	 * See commit 59bfb0f3189b ("Add support for the Mares Matrix")
-	 * for details.
-	 */
-	device->splitcommand = 1;
+	memset (device->cache, 0, sizeof (device->cache));
+	device->available = 0;
+	device->offset = 0;
 
 	// Set the serial communication protocol (115200 8E1).
 	status = dc_iostream_configure (device->iostream, 115200, 8, DC_PARITY_EVEN, DC_STOPBITS_ONE, DC_FLOWCONTROL_NONE);
@@ -282,7 +505,7 @@ mares_iconhd_device_open (dc_device_t **out, dc_context_t *context, dc_iostream_
 	dc_iostream_purge (device->iostream, DC_DIRECTION_ALL);
 
 	// Send the version command.
-	unsigned char command[] = {0xC2, 0x67};
+	unsigned char command[] = {CMD_VERSION, CMD_VERSION ^ XOR};
 	status = mares_iconhd_transfer (device, command, sizeof (command),
 		device->version, sizeof (device->version));
 	if (status != DC_STATUS_SUCCESS) {
@@ -292,6 +515,20 @@ mares_iconhd_device_open (dc_device_t **out, dc_context_t *context, dc_iostream_
 	// Autodetect the model using the version packet.
 	device->model = mares_iconhd_get_model (device);
 
+	// Read the size of the flash memory.
+	unsigned int memsize = 0;
+	if (device->model == QUAD) {
+		unsigned char cmd_flash[] = {CMD_FLASHSIZE, CMD_FLASHSIZE ^ XOR};
+		unsigned char rsp_flash[4] = {0};
+		status = mares_iconhd_transfer (device, cmd_flash, sizeof (cmd_flash), rsp_flash, sizeof (rsp_flash));
+		if (status != DC_STATUS_SUCCESS) {
+			WARNING (context, "Failed to read the flash memory size.");
+		} else {
+			memsize = array_uint32_le (rsp_flash);
+			DEBUG (context, "Flash memory size is %u bytes.", memsize);
+		}
+	}
+
 	// Load the correct memory layout.
 	switch (device->model) {
 	case MATRIX:
@@ -303,15 +540,27 @@ mares_iconhd_device_open (dc_device_t **out, dc_context_t *context, dc_iostream_
 	case NEMOWIDE2:
 	case SMART:
 	case SMARTAPNEA:
-	case QUAD:
 		device->layout = &mares_nemowide2_layout;
 		device->packetsize = 256;
 		break;
+	case QUAD:
+		if (memsize > 0x40000) {
+			device->layout = &mares_iconhd_layout;
+		} else {
+			device->layout = &mares_nemowide2_layout;
+		}
+		device->packetsize = 256;
+		break;
 	case QUADAIR:
 	case SMARTAIR:
 		device->layout = &mares_iconhdnet_layout;
 		device->packetsize = 256;
 		break;
+	case GENIUS:
+		device->layout = &mares_iconhdnet_layout;
+		device->packetsize = 256;
+		device->fingerprint_size = 4;
+		break;
 	case ICONHDNET:
 		device->layout = &mares_iconhdnet_layout;
 		device->packetsize = 4096;
@@ -323,26 +572,6 @@ mares_iconhd_device_open (dc_device_t **out, dc_context_t *context, dc_iostream_
 		break;
 	}
 
-	if (dc_iostream_get_transport(device->iostream) == DC_TRANSPORT_BLE) {
-		/*
-		 * Don't ask for larger amounts of data with the BLE
-		 * transport - it will fail.  I suspect there is a buffer
-		 * overflow in the BlueLink Pro dongle when bluetooth is
-		 * slower than the serial protocol that the dongle talks to
-		 * the dive computer.
-		 */
-		if (device->packetsize > 128)
-			device->packetsize = 128;
-		/*
-		 * With BLE, don't wait for ACK before sending the arguments
-		 * to a command.
-		 *
-		 * There is some timing issue that makes that take too long
-		 * and causes the command to be aborted.
-		 */
-		device->splitcommand = 0;
-	}
-
 	*out = (dc_device_t *) device;
 
 	return DC_STATUS_SUCCESS;
@@ -358,13 +587,13 @@ mares_iconhd_device_set_fingerprint (dc_device_t *abstract, const unsigned char
 {
 	mares_iconhd_device_t *device = (mares_iconhd_device_t *) abstract;
 
-	if (size && size != sizeof (device->fingerprint))
+	if (size && size != device->fingerprint_size)
 		return DC_STATUS_INVALIDARGS;
 
 	if (size)
-		memcpy (device->fingerprint, data, sizeof (device->fingerprint));
+		memcpy (device->fingerprint, data, device->fingerprint_size);
 	else
-		memset (device->fingerprint, 0, sizeof (device->fingerprint));
+		memset (device->fingerprint, 0, device->fingerprint_size);
 
 	return DC_STATUS_SUCCESS;
 }
@@ -384,7 +613,7 @@ mares_iconhd_device_read (dc_device_t *abstract, unsigned int address, unsigned
 			len = device->packetsize;
 
 		// Read the packet.
-		unsigned char command[] = {0xE7, 0x42,
+		unsigned char command[] = {CMD_READ, CMD_READ ^ XOR,
 			(address      ) & 0xFF,
 			(address >>  8) & 0xFF,
 			(address >> 16) & 0xFF,
@@ -427,48 +656,19 @@ mares_iconhd_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 		dc_buffer_get_size (buffer), device->packetsize);
 }
 
-
 static dc_status_t
-mares_iconhd_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata)
+mares_iconhd_device_foreach_raw (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata)
 {
 	dc_status_t rc = DC_STATUS_SUCCESS;
 	mares_iconhd_device_t *device = (mares_iconhd_device_t *) abstract;
 
-	if (!ISINSTANCE (abstract))
-		return DC_STATUS_INVALIDARGS;
-
 	const mares_iconhd_layout_t *layout = device->layout;
 
 	// Enable progress notifications.
 	dc_event_progress_t progress = EVENT_PROGRESS_INITIALIZER;
-	progress.maximum = layout->rb_profile_end - layout->rb_profile_begin + 4;
+	progress.maximum = layout->rb_profile_end - layout->rb_profile_begin;
 	device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
 
-	// Emit a vendor event.
-	dc_event_vendor_t vendor;
-	vendor.data = device->version;
-	vendor.size = sizeof (device->version);
-	device_event_emit (abstract, DC_EVENT_VENDOR, &vendor);
-
-	// Read the serial number.
-	unsigned char serial[4] = {0};
-	rc = mares_iconhd_device_read (abstract, 0x0C, serial, sizeof (serial));
-	if (rc != DC_STATUS_SUCCESS) {
-		ERROR (abstract->context, "Failed to read the memory.");
-		return rc;
-	}
-
-	// Update and emit a progress event.
-	progress.current += sizeof (serial);
-	device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
-
-	// Emit a device info event.
-	dc_event_devinfo_t devinfo;
-	devinfo.model = device->model;
-	devinfo.firmware = 0;
-	devinfo.serial = array_uint32_le (serial);
-	device_event_emit (abstract, DC_EVENT_DEVINFO, &devinfo);
-
 	// Get the model code.
 	unsigned int model = device->model;
 
@@ -645,3 +845,113 @@ mares_iconhd_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback,
 
 	return rc;
 }
+
+static dc_status_t
+mares_iconhd_device_foreach_object (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata)
+{
+	dc_status_t rc = DC_STATUS_SUCCESS;
+	mares_iconhd_device_t *device = (mares_iconhd_device_t *) abstract;
+
+	// Enable progress notifications.
+	dc_event_progress_t progress = EVENT_PROGRESS_INITIALIZER;
+	device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
+
+	// Allocate memory for the dives.
+	dc_buffer_t *buffer = dc_buffer_new (4096);
+	if (buffer == NULL) {
+		ERROR (abstract->context, "Failed to allocate memory.");
+		return DC_STATUS_NOMEMORY;
+	}
+
+	// Read the number of dives.
+	rc = mares_iconhd_read_object (device, NULL, buffer, OBJ_LOGBOOK, OBJ_LOGBOOK_COUNT);
+	if (rc != DC_STATUS_SUCCESS) {
+		ERROR (abstract->context, "Failed to read the number of dives.");
+		dc_buffer_free (buffer);
+		return rc;
+	}
+
+	if (dc_buffer_get_size (buffer) < 2) {
+		ERROR (abstract->context, "Unexpected number of bytes received (" DC_PRINTF_SIZE ").",
+			dc_buffer_get_size (buffer));
+		dc_buffer_free (buffer);
+		return DC_STATUS_PROTOCOL;
+	}
+
+	// Get the number of dives.
+	unsigned int ndives = array_uint16_le (dc_buffer_get_data(buffer));
+
+	// Update and emit a progress event.
+	progress.current = 1 * NSTEPS;
+	progress.maximum = (1 + ndives * 2) * NSTEPS;
+	device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
+
+	// Download the dives.
+	for (unsigned int i = 0; i < ndives; ++i) {
+		// Erase the buffer.
+		dc_buffer_clear (buffer);
+
+		// Read the dive header.
+		rc = mares_iconhd_read_object (device, &progress, buffer, OBJ_DIVE + i, OBJ_DIVE_HEADER);
+		if (rc != DC_STATUS_SUCCESS) {
+			ERROR (abstract->context, "Failed to read the dive header.");
+			break;
+		}
+
+		// Check the fingerprint data.
+		if (memcmp (dc_buffer_get_data (buffer) + 0x08, device->fingerprint, device->fingerprint_size) == 0) {
+			INFO (abstract->context, "Stopping due to detecting a matching fingerprint");
+			break;
+		}
+
+		// Read the dive data.
+		rc = mares_iconhd_read_object (device, &progress, buffer, OBJ_DIVE + i, OBJ_DIVE_DATA);
+		if (rc != DC_STATUS_SUCCESS) {
+			ERROR (abstract->context, "Failed to read the dive data.");
+			break;
+		}
+
+		const unsigned char *data = dc_buffer_get_data (buffer);
+		if (callback && !callback (data, dc_buffer_get_size (buffer), data + 0x08, device->fingerprint_size, userdata)) {
+			break;
+		}
+	}
+
+	dc_buffer_free(buffer);
+
+	return rc;
+}
+
+static dc_status_t
+mares_iconhd_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata)
+{
+	dc_status_t rc = DC_STATUS_SUCCESS;
+	mares_iconhd_device_t *device = (mares_iconhd_device_t *) abstract;
+
+	// Emit a vendor event.
+	dc_event_vendor_t vendor;
+	vendor.data = device->version;
+	vendor.size = sizeof (device->version);
+	device_event_emit (abstract, DC_EVENT_VENDOR, &vendor);
+
+	// Read the serial number.
+	unsigned char serial[4] = {0};
+	rc = mares_iconhd_device_read (abstract, 0x0C, serial, sizeof (serial));
+	if (rc != DC_STATUS_SUCCESS) {
+		ERROR (abstract->context, "Failed to read the memory.");
+		return rc;
+	}
+
+	// Emit a device info event.
+	dc_event_devinfo_t devinfo;
+	devinfo.model = device->model;
+	devinfo.firmware = 0;
+	devinfo.serial = array_uint32_le (serial);
+	device_event_emit (abstract, DC_EVENT_DEVINFO, &devinfo);
+
+	if (device->model == GENIUS) {
+		return mares_iconhd_device_foreach_object (abstract, callback, userdata);
+	} else {
+		return mares_iconhd_device_foreach_raw (abstract, callback, userdata);
+	}
+}
