@@ -76,6 +76,7 @@
 #define F11A        0x4549
 #define OCI         0x454B
 #define A300CS      0x454C
+#define TALIS       0x454E
 #define MUNDIAL3    0x4550
 #define PROPLUSX    0x4552
 #define F10B        0x4553
@@ -91,6 +92,8 @@
 #define I300C       0x4648
 #define I100        0x464E
 #define I770R       0x4651
+#define I550C       0x4652
+#define GEO40       0x4653
 
 #define NORMAL   0
 #define GAUGE    1
@@ -182,8 +185,10 @@ oceanic_atom2_parser_create (dc_parser_t **out, dc_context_t *context, unsigned
 		model == I450T || model == I750TC ||
 		model == I770R) {
 		parser->headersize = 5 * PAGESIZE;
-	} else if (model == PROPLUSX) {
+	} else if (model == PROPLUSX || model == GEO40) {
 		parser->headersize = 3 * PAGESIZE;
+	} else if (model == I550C) {
+		parser->headersize = 5 * PAGESIZE / 2;
 	}
 
 	parser->serial = serial;
@@ -259,6 +264,7 @@ oceanic_atom2_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetim
 		case A300AI:
 		case OCI:
 		case I550:
+		case I550C:
 		case VISION:
 		case XPAIR:
 			datetime->year   = ((p[5] & 0xE0) >> 5) + ((p[7] & 0xE0) >> 2) + 2000;
@@ -292,6 +298,7 @@ oceanic_atom2_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetim
 		case AMPHOS:
 		case AMPHOSAIR:
 		case VOYAGER2G:
+		case TALIS:
 			datetime->year   = (p[3] & 0x1F) + 2000;
 			datetime->month  = (p[7] & 0xF0) >> 4;
 			datetime->day    = ((p[3] & 0x80) >> 3) + ((p[5] & 0xF0) >> 4);
@@ -323,6 +330,7 @@ oceanic_atom2_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetim
 		case I450T:
 		case I750TC:
 		case PROPLUSX:
+		case GEO40:
 		case I770R:
 			datetime->year   = (p[10]) + 2000;
 			datetime->month  = (p[8]);
@@ -462,7 +470,7 @@ oceanic_atom2_parser_cache (oceanic_atom2_parser_t *parser)
 	} else if (parser->model == ZEN) {
 		o2_offset = header + 4;
 		ngasmixes = 2;
-	} else if (parser->model == PROPLUSX) {
+	} else if (parser->model == PROPLUSX || parser->model == GEO40) {
 		o2_offset = 0x24;
 		ngasmixes = 4;
 	} else if (parser->model == I770R) {
@@ -661,7 +669,8 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 		unsigned int idx = 0x17;
 		if (parser->model == A300CS || parser->model == VTX ||
 			parser->model == I450T || parser->model == I750TC ||
-			parser->model == PROPLUSX || parser->model == I770R)
+			parser->model == PROPLUSX || parser->model == I770R ||
+			parser->model == GEO40)
 			idx = 0x1f;
 		switch (data[idx] & 0x03) {
 		case 0:
@@ -717,7 +726,7 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 		parser->model == TX1 || parser->model == A300CS ||
 		parser->model == VTX || parser->model == I450T ||
 		parser->model == I750TC || parser->model == PROPLUSX ||
-		parser->model == I770R) {
+		parser->model == GEO40 || parser->model == I770R) {
 		samplesize = PAGESIZE;
 	}
 
@@ -731,7 +740,7 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 		parser->model == GEO || parser->model == GEO20 ||
 		parser->model == MANTA || parser->model == I300 ||
 		parser->model == I200 || parser->model == I100 ||
-		parser->model == I300C) {
+		parser->model == I300C || parser->model == TALIS) {
 		have_pressure = 0;
 	}
 
@@ -881,6 +890,8 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 					parser->model == ELEMENT2 || parser->model == MANTA ||
 					parser->model == ZEN) {
 					temperature = data[offset + 6];
+				} else if (parser->model == TALIS) {
+					temperature = data[offset + 7];
 				} else if (parser->model == GEO20 || parser->model == VEO20 ||
 					parser->model == VEO30 || parser->model == OC1A ||
 					parser->model == OC1B || parser->model == OC1C ||
@@ -898,12 +909,12 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 					temperature = ((data[offset + 7] & 0xF0) >> 4) | ((data[offset + 7] & 0x0C) << 2) | ((data[offset + 5] & 0x0C) << 4);
 				} else if (parser->model == A300CS || parser->model == VTX ||
 					parser->model == I750TC || parser->model == PROPLUSX ||
-					parser->model == I770R) {
+					parser->model == GEO40 || parser->model == I770R) {
 					temperature = data[offset + 11];
 				} else {
 					unsigned int sign;
 					if (parser->model == DG03 || parser->model == PROPLUS3 ||
-						parser->model == I550)
+						parser->model == I550 || parser->model == I550C)
 						sign = (~data[offset + 5] & 0x04) >> 2;
 					else if (parser->model == VOYAGER2G || parser->model == AMPHOS ||
 						parser->model == AMPHOSAIR || parser->model == ZENAIR)
@@ -935,11 +946,13 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 					parser->model == ZENAIR ||parser->model == A300AI ||
 					parser->model == DG03 || parser->model == PROPLUS3 ||
 					parser->model == AMPHOSAIR || parser->model == I550 ||
-					parser->model == VISION || parser->model == XPAIR)
+					parser->model == VISION || parser->model == XPAIR ||
+					parser->model == I550C)
 					pressure = (((data[offset + 0] & 0x03) << 8) + data[offset + 1]) * 5;
 				else if (parser->model == TX1 || parser->model == A300CS ||
 					parser->model == VTX || parser->model == I750TC ||
-					parser->model == PROPLUSX || parser->model == I770R)
+					parser->model == PROPLUSX || parser->model == GEO40 ||
+					parser->model == I770R)
 					pressure = array_uint16_le (data + offset + 4);
 				else
 					pressure -= data[offset + 1];
@@ -988,8 +1001,10 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 			unsigned int have_deco = 0;
 			unsigned int decostop = 0, decotime = 0;
 			if (parser->model == A300CS || parser->model == VTX ||
-				parser->model == I450T || parser->model == I750TC ||
-				parser->model == PROPLUSX || parser->model == I770R) {
+				parser->model == I750TC ||
+				parser->model == PROPLUSX ||
+				parser->model == GEO40 ||
+				parser->model == I770R) {
 				decostop = (data[offset + 15] & 0x70) >> 4;
 				decotime = array_uint16_le(data + offset + 6) & 0x03FF;
 				have_deco = 1;
@@ -1002,14 +1017,16 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 				decotime = array_uint16_le(data + offset + 6);
 				have_deco = 1;
 			} else if (parser->model == ATOM31 || parser->model == VISION ||
-				parser->model == XPAIR || parser->model == I550) {
+				parser->model == XPAIR || parser->model == I550 ||
+				parser->model == I550C) {
 				decostop = (data[offset + 5] & 0xF0) >> 4;
 				decotime = array_uint16_le(data + offset + 4) & 0x03FF;
 				have_deco = 1;
 			} else if (parser->model == I200 || parser->model == I300 ||
 				parser->model == OC1A || parser->model == OC1B ||
 				parser->model == OC1C || parser->model == OCI ||
-				parser->model == I100 || parser->model == I300C) {
+				parser->model == I100 || parser->model == I300C ||
+				parser->model == I450T) {
 				decostop = (data[offset + 7] & 0xF0) >> 4;
 				decotime = array_uint16_le(data + offset + 6) & 0x0FFF;
 				have_deco = 1;
@@ -1034,11 +1051,11 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 			} else if (parser->model == I450T || parser->model == OC1A ||
 				parser->model == OC1B || parser->model == OC1C ||
 				parser->model == OCI || parser->model == PROPLUSX ||
-				parser->model == I770R) {
+				parser->model == GEO40 || parser->model == I770R) {
 				rbt = array_uint16_le(data + offset + 8) & 0x01FF;
 				have_rbt = 1;
 			} else if (parser->model == VISION || parser->model == XPAIR ||
-				parser->model == I550) {
+				parser->model == I550 || parser->model == I550C) {
 				rbt = array_uint16_le(data + offset + 6) & 0x03FF;
 				have_rbt = 1;
 			}
